package editor;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;

import milionario.Domanda;

/**
 * Editor di testo per le domande. Legge solo i file .urb
 * 
 * @author Andrea Urbini
 * @version 2.6
 */

public class Editor2 extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JTextComponent editor;
	private JFileChooser fileChooser;
	private JMenuItem OpenMenuItem;
	private JMenuItem SaveMenuItem;
	private JMenuItem CutMenuItem;
	private JMenuItem CopyMenuItem;
	private JMenuItem PasteMenuItem;
	private JMenuItem ExitMenuItem;
	private JMenuItem InfoMenuItem;
	private JMenuItem FindMenuItem;
	private JMenuItem CompileMenuItem;
	private JMenuItem HelpMenuItem;
	private JButton OpenButton;
	private JButton SaveButton;
	private JButton CutButton;
	private JButton CopyButton;
	private JButton PasteButton;
	private JButton FindButton;
	private JButton CompileButton;
	private JButton HelpButton;
	private String fileName;
	private final String PATH = "data/";
	private final String FRAME_TITLE = "Editor delle domande";
	private HelpFrame helpFrame;
	private final Icon WARNING_ICON = new ImageIcon(getClass().getResource(
			"/data/warning.png"));
	private final Icon ERROR_ICON = new ImageIcon(getClass().getResource(
			"/data/error.png"));
	private final Icon INFO_ICON = new ImageIcon(getClass().getResource(
			"/data/info.png"));

	/**
	 * Costruttore
	 * 
	 */

	public Editor2() {
		super();
		setupFrame();
		setupComponents();
		buildGUI();
		registerListeners();
		setVisible(true);
	}

	/**
	 * 
	 */
	public void actionPerformed(ActionEvent ae) {
		if (ae.getSource().equals(OpenButton)
				|| ae.getSource().equals(OpenMenuItem)) {
			try {
				open();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, "Errore di Input o Output",
						"I/O Error", JOptionPane.ERROR_MESSAGE, ERROR_ICON);
			}
		} else if (ae.getSource().equals(SaveButton)
				|| ae.getSource().equals(SaveMenuItem)) {
			try {
				save();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, "Errore di Input o Output",
						"I/O Error", JOptionPane.ERROR_MESSAGE, ERROR_ICON);
			}
		} else if (ae.getSource().equals(CutButton)
				|| ae.getSource().equals(CutMenuItem)) {
			cut();
		} else if (ae.getSource().equals(CopyButton)
				|| ae.getSource().equals(CopyMenuItem)) {
			copy();
		} else if (ae.getSource().equals(PasteButton)
				|| ae.getSource().equals(PasteMenuItem)) {
			paste();
		} else if (ae.getSource().equals(FindButton)
				|| ae.getSource().equals(FindMenuItem)) {
			String text = JOptionPane.showInputDialog(this,
					"Inserisci la stringa da cercare:", "");
			find(text);
		} else if (ae.getSource().equals(InfoMenuItem)) {
			JOptionPane
					.showMessageDialog(
							this,
							"UrboPad\nby Andrea Urbini 2006\nhttp://www.urbo.altervista.org",
							"Informazioni su...",
							JOptionPane.INFORMATION_MESSAGE, INFO_ICON);
		} else if ((ae.getSource().equals(ExitMenuItem))) {
			System.exit(0);
		} else if (ae.getSource().equals(CompileButton)
				|| ae.getSource().equals(CompileMenuItem)) {
			if (getEditor().getText().length() != 0) {
				try {
					compile();
				} catch (IOException ioe) {
					ioe.printStackTrace();
					JOptionPane.showMessageDialog(this,
							"Errore di Input o Output", "I/O Exception",
							JOptionPane.ERROR_MESSAGE, ERROR_ICON);
				} catch (NumberFormatException nfe) {
					nfe.printStackTrace();
					JOptionPane
							.showMessageDialog(this,
									"Errore sul formato dei numeri "
											+ nfe.getMessage(),
									"Number Format Exception",
									JOptionPane.ERROR_MESSAGE, ERROR_ICON);
				}
			} else {
				JOptionPane.showMessageDialog(this,
						"Prima di compilare bisogna aprire un file .urb!",
						"Errore", JOptionPane.WARNING_MESSAGE, WARNING_ICON);
			}
		} else if (ae.getSource().equals(HelpButton)
				|| ae.getSource().equals(HelpMenuItem)) {
			helpFrame.setVisible(true);
		}
	}

	protected void buildButtons() {
		OpenButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/open.png")));
		OpenButton.setToolTipText("Apri");
		SaveButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/save.png")));
		SaveButton.setToolTipText("Salva");
		CutButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/cut.png")));
		CutButton.setToolTipText("Taglia");
		CopyButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/copy.png")));
		CopyButton.setToolTipText("Copia");
		PasteButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/paste.png")));
		PasteButton.setToolTipText("Incolla");
		FindButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/find.png")));
		FindButton.setToolTipText("Cerca");
		CompileButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/compile.png")));
		CompileButton.setToolTipText("Compila");
		HelpButton = new JButton(new ImageIcon(getClass().getResource(
				"/data/help.png")));
		HelpButton.setToolTipText("Aiuto");
	}

	/**
	 * Crea un oggetto Domanda
	 * 
	 * @param correctTokens
	 *            array con le stringe necessarie alla creazione della domanda
	 * @return la domanda generata
	 */
	private Domanda buildDomanda(String[] correctTokens)
			throws NumberFormatException {
		int idNumber = Integer.parseInt(correctTokens[0]);
		String testo = correctTokens[1];
		String rispA = correctTokens[2];
		String rispB = correctTokens[3];
		String rispC = correctTokens[4];
		String rispD = correctTokens[5];
		int rispGiusta = Integer.parseInt(correctTokens[6]);
		int[] perc = new int[] { Integer.parseInt(correctTokens[7]),
				Integer.parseInt(correctTokens[8]),
				Integer.parseInt(correctTokens[9]),
				Integer.parseInt(correctTokens[10]), 0 };
		return new Domanda(idNumber, testo, rispA, rispB, rispC, rispD,
				rispGiusta, perc);
	}

	protected void buildGUI() {
		setJMenuBar(createMenuBar());
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(BorderLayout.NORTH, createToolBar());
		getContentPane().add(BorderLayout.CENTER, new JScrollPane(getEditor()));
	}

	protected void buildMenuItems() {
		OpenMenuItem = new JMenuItem("Apri", new ImageIcon(getClass()
				.getResource("/data/open.png")));
		OpenMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				Event.CTRL_MASK, true));
		SaveMenuItem = new JMenuItem("Salva con nome", new ImageIcon(getClass()
				.getResource("/data/save.png")));
		SaveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.CTRL_MASK, true));
		CutMenuItem = new JMenuItem("Taglia", new ImageIcon(getClass()
				.getResource("/data/cut.png")));
		CutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
				Event.CTRL_MASK, true));
		CopyMenuItem = new JMenuItem("Copia", new ImageIcon(getClass()
				.getResource("/data/copy.png")));
		CopyMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
				Event.CTRL_MASK, true));
		PasteMenuItem = new JMenuItem("Incolla", new ImageIcon(getClass()
				.getResource("/data/paste.png")));
		PasteMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
				Event.CTRL_MASK, true));
		ExitMenuItem = new JMenuItem("Esci", new ImageIcon(getClass()
				.getResource("/data/exit.png")));
		InfoMenuItem = new JMenuItem("Informazioni su...", new ImageIcon(
				getClass().getResource("/data/info.png")));
		FindMenuItem = new JMenuItem("Cerca", new ImageIcon(getClass()
				.getResource("/data/find.png")));
		FindMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,
				Event.CTRL_MASK, true));
		CompileMenuItem = new JMenuItem("Compila", new ImageIcon(getClass()
				.getResource("/data/compile.png")));
		HelpMenuItem = new JMenuItem("Aiuto", new ImageIcon(getClass()
				.getResource("/data/help.png")));
	}

	/**
	 * Crea il file criptato delle domande.
	 * 
	 * @throws IOException
	 */
	protected void compile() throws IOException, NumberFormatException {
		ArrayList<Domanda> vettoreDomande = new ArrayList<Domanda>();
		int[] domandePerLivello = new int[3];
		String[] lines = new String[11];
		BufferedReader br = new BufferedReader(new FileReader(PATH
				+ getFileName()));
		int i = 0;
		while (br.ready()) {
			lines[i] = br.readLine();
			if (i == 10) {
				Domanda domanda = buildDomanda(lines);
				domandePerLivello = getDomandePerLivello(domanda,
						domandePerLivello);
				vettoreDomande.add(domanda);
				i = 0;
			} else {
				i++;
			}
		}
		FileOutputStream fos1 = new FileOutputStream(PATH + "domande.urb.cry");
		ObjectOutputStream oos1 = new ObjectOutputStream(fos1);
		oos1.writeObject(vettoreDomande);
		oos1.close();
		FileOutputStream fos2 = new FileOutputStream(PATH + "domande.urb.nbr");
		ObjectOutputStream oos2 = new ObjectOutputStream(fos2);
		oos2.writeObject(domandePerLivello);
		oos2.close();
		JOptionPane
				.showMessageDialog(
						this,
						"Sono stati creati i seguenti files:\n- domande.urb.cry\n- domande.urb.nbr",
						"Compilazione avvenuta con successo!",
						JOptionPane.INFORMATION_MESSAGE, INFO_ICON);
	}

	protected void copy() {
		getEditor().copy();
	}

	protected JTextComponent createEditor() {
		return new JTextArea();
	}

	protected JFileChooser createFileChooser() {
		JFileChooser f = new JFileChooser("./" + PATH);
		f.addChoosableFileFilter(new FiltroFile());
		return f;
	}

	/**
	 * 
	 * @return
	 */
	protected JMenuBar createMenuBar() {
		JMenu fileMenu = new JMenu("File");
		fileMenu.add(OpenMenuItem);
		fileMenu.add(SaveMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(ExitMenuItem);
		JMenu editMenu = new JMenu("Modifica");
		editMenu.add(CutMenuItem);
		editMenu.add(CopyMenuItem);
		editMenu.add(PasteMenuItem);
		editMenu.addSeparator();
		editMenu.add(FindMenuItem);
		JMenu runMenu = new JMenu("Run");
		runMenu.add(CompileMenuItem);
		JMenu infoMenu = new JMenu("?");
		infoMenu.add(InfoMenuItem);
		infoMenu.add(HelpMenuItem);
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(runMenu);
		menuBar.add(infoMenu);
		return menuBar;
	}

	/**
	 * 
	 * @return
	 */
	protected JToolBar createToolBar() {
		JToolBar toolbar = new JToolBar();
		toolbar.add(OpenButton);
		toolbar.add(SaveButton);
		toolbar.addSeparator();
		toolbar.add(CutButton);
		toolbar.add(CopyButton);
		toolbar.add(PasteButton);
		toolbar.addSeparator();
		toolbar.add(FindButton);
		toolbar.addSeparator();
		toolbar.add(CompileButton);
		toolbar.addSeparator();
		toolbar.add(HelpButton);
		return toolbar;
	}

	/**
	 * 
	 * 
	 */
	protected void cut() {
		getEditor().cut();
	}

	/**
	 * Cerca una parola nel testo visualizzato nel pannello principale.
	 * 
	 * @param toFind
	 *            la parola da cercare
	 */
	protected void find(String toFind) {
		int length = toFind.length();
		int index = -1;
		int risp = JOptionPane.YES_OPTION;
		do {
			index = findSubSequence(toFind, index + 1, length);
			if (index == -1) {
				JOptionPane.showMessageDialog(this, "Stringa \"" + toFind
						+ "\" non trovata.", "Errore",
						JOptionPane.WARNING_MESSAGE, WARNING_ICON);
				break;
			} else {
				editor.requestFocus();
				editor.setCaretPosition(index);
				editor.setSelectionStart(index);
				editor.setSelectionEnd(index + length);
				risp = JOptionPane.showConfirmDialog(this,
						"Vuoi cercare ancora?", "", JOptionPane.YES_NO_OPTION);
			}
		} while (risp == JOptionPane.YES_OPTION);
	}

	private int findSubSequence(String toFind, int index, int length) {
		do {
			try {
				String text = editor.getText(index, length);
				if (toFind.equals(text)) {
					break;
				}
				index++;
			} catch (BadLocationException ble) {
				return -1;
			}
		} while (index > -2);
		return index;
	}

	/**
	 * Controlla il livello della domanda e restituisce il numero di domande che
	 * ci sono in ogni livello.
	 * 
	 * @param domanda
	 *            domanda da controllare
	 * @param domandePerLivello
	 *            array contenente il numero di domande per ogni livello
	 * @return array contenente il numero di domande per ogni livello
	 */
	protected int[] getDomandePerLivello(Domanda domanda,
			int[] domandePerLivello) {
		if (domanda.getIdNumber() < 20000) {
			domandePerLivello[0]++;
		} else if (domanda.getIdNumber() < 30000) {
			domandePerLivello[1]++;
		} else {
			domandePerLivello[2]++;
		}
		return domandePerLivello;
	}

	protected JTextComponent getEditor() {
		return editor;
	}

	protected JFileChooser getFileChooser() {
		return fileChooser;
	}

	protected String getFileName() {
		return fileName;
	}

	protected String loadManual() {
		String manuale = "";
		InputStreamReader isr = new InputStreamReader(getClass()
				.getResourceAsStream("/data/manuale_editor.txt"));
		BufferedReader br = new BufferedReader(isr);
		try {
			while (br.ready()) {
				manuale = manuale + br.readLine() + "\n";
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return "Errore nel caricamento del manuale.";
		}
		try {
			isr.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return "Errore nel caricamento del manuale.";
		}
		return manuale;
	}

	/**
	 * 
	 * @throws IOException
	 */
	protected void open() throws IOException {
		int response = getFileChooser().showOpenDialog(this);
		if (response == JFileChooser.APPROVE_OPTION) {
			File f = getFileChooser().getSelectedFile();
			Reader in = new FileReader(f);
			fileName = f.getName();
			editor.read(in, null);
			setTitle(FRAME_TITLE + " : " + fileName);
		}
	}

	protected void paste() {
		getEditor().paste();
	}

	/**
	 * 
	 * 
	 */
	protected void registerListeners() {
		OpenMenuItem.addActionListener(this);
		SaveMenuItem.addActionListener(this);
		CutMenuItem.addActionListener(this);
		CopyMenuItem.addActionListener(this);
		PasteMenuItem.addActionListener(this);
		ExitMenuItem.addActionListener(this);
		InfoMenuItem.addActionListener(this);
		FindMenuItem.addActionListener(this);
		CompileMenuItem.addActionListener(this);
		HelpMenuItem.addActionListener(this);

		OpenButton.addActionListener(this);
		SaveButton.addActionListener(this);
		CutButton.addActionListener(this);
		CopyButton.addActionListener(this);
		PasteButton.addActionListener(this);
		FindButton.addActionListener(this);
		CompileButton.addActionListener(this);
		HelpButton.addActionListener(this);
	}

	/**
	 * 
	 * @throws IOException
	 */
	protected void save() throws IOException {
		int response = getFileChooser().showSaveDialog(this);
		if (response == JFileChooser.APPROVE_OPTION) {
			File f = getFileChooser().getSelectedFile();
			Writer out = new FileWriter(f);
			fileName = f.getName();
			setTitle(FRAME_TITLE + " : " + fileName);
			editor.write(out);
		}
	}

	protected void setupComponents() {
		editor = createEditor();
		fileChooser = createFileChooser();
		helpFrame = new HelpFrame(loadManual());
		buildMenuItems();
		buildButtons();

	}

	protected void setupFrame() {
		setTitle(FRAME_TITLE);
		setSize(800, 600);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
		setLocation((int) (d.getWidth() - getWidth()) / 2,
				(int) (d.getHeight() - getHeight()) / 2);
		setVisible(true);
	}
}
