package milionario;

import java.io.Serializable;

/**
 *Rappresenta l'astrazione di una domanda.
 * 
 *@author Andrea Urbini
 *@version 2.0
 */

public class Domanda implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1926268322581883265L;
	private String testo, rispostaA, rispostaB, rispostaC, rispostaD;
	private int rispostaGiusta;
	private int[] percentuali;
	private int[] rispElim;
	private int idNumber;

	/**
	 * 
	 * @param id
	 *            numero identificativo della domanda
	 * @param t
	 *            testo della domanda
	 * @param a
	 *            risposta A
	 * @param b
	 *            risposta B
	 * @param c
	 *            risposta C
	 * @param d
	 *            risposta D
	 * @param rispGiusta
	 *            numero della risposta giusta: 0 = A; 1 = B; 2 = C; 3 = D;
	 * @param perc
	 *            array che contiene le percentuali delle votazioni del pubblico
	 */
	public Domanda(int id, String t, String a, String b, String c, String d,
			int rispGiusta, int[] perc) {
		idNumber = id;
		testo = t;
		rispostaA = a;
		rispostaB = b;
		rispostaC = c;
		rispostaD = d;
		rispostaGiusta = rispGiusta;
		percentuali = perc;
	}

	/**
	 * 
	 * @return
	 */
	public int getIdNumber() {
		return idNumber;
	}

	/**
	 * 
	 * @return
	 */
	public int[] getPercentuali() {
		return percentuali;
	}

	/**
	 *Genera casualmente le due risposte da eliminare nell'aiuto del computer.
	 * 
	 * @return vettore con i numeri delle risposte da eliminare
	 */
	public int[] getRispElimComp() {
		int n = (int) (Math.random() * 3);
		switch (rispostaGiusta) {
		case 1: {
			if (n == 1) {
				rispElim = new int[] { 2, 3 };
				percentuali[4] = 5;
			} else if (n == 2) {
				rispElim = new int[] { 2, 4 };
				percentuali[4] = 6;
			} else {
				rispElim = new int[] { 3, 4 };
				percentuali[4] = 7;
			}
			break;
		}
		case 2: {
			if (n == 1) {
				rispElim = new int[] { 1, 3 };
				percentuali[4] = 4;
			} else if (n == 2) {
				rispElim = new int[] { 1, 4 };
				percentuali[4] = 8;
			} else {
				rispElim = new int[] { 3, 4 };
				percentuali[4] = 7;
			}
			break;
		}
		case 3: {
			if (n == 1) {
				rispElim = new int[] { 1, 2 };
				percentuali[4] = 3;
			} else if (n == 2) {
				rispElim = new int[] { 2, 4 };
				percentuali[4] = 6;
			} else {
				rispElim = new int[] { 1, 4 };
				percentuali[4] = 8;
			}
			break;
		}
		case 4: {
			if (n == 1) {
				rispElim = new int[] { 1, 2 };
				percentuali[4] = 3;
			} else if (n == 2) {
				rispElim = new int[] { 2, 3 };
				percentuali[4] = 5;
			} else {
				rispElim = new int[] { 1, 3 };
				percentuali[4] = 4;
			}
			break;
		}
		}
		return rispElim;
	}

	/**
	 * 
	 * @return
	 */
	public String getRispostaA() {
		return rispostaA;
	}

	/**
	 * 
	 * @return
	 */
	public String getRispostaB() {
		return rispostaB;
	}

	/**
	 * 
	 * @return
	 */
	public String getRispostaC() {
		return rispostaC;
	}

	/**
	 * 
	 * @return
	 */
	public String getRispostaD() {
		return rispostaD;
	}

	/**
	 * 
	 * @return
	 */
	public int getRispostaGiusta() {
		return rispostaGiusta;
	}

	/**
	 * 
	 * @return
	 */
	public String getTesto() {
		return testo;
	}
}