package milionario;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.InputStream;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * Pannello in cui viene visualizzata la domanda. Gestisce anche il mouse.
 * 
 * @author Andrea Urbini
 * @version 6.5
 */

public class DomandaPanel extends JPanel implements MouseListener {

	private static final long serialVersionUID = 1L;
	private JTextArea domandaField, rispostaA, rispostaB, rispostaC, rispostaD;
	private FlagRegister flag;
	private int rispostaSelezionata = 0;
	private int rispostaGiusta;
	private final Font FONT = new Font("SansSerif", Font.PLAIN, 11);
	private Clip ol;
	private Image img;

	/**
	 * 
	 * @param flag
	 */
	public DomandaPanel(FlagRegister flag) {
		super();
		setLayout(null);
		this.flag = flag;
		setupComponents();
		add(domandaField);
		add(rispostaA);
		add(rispostaB);
		add(rispostaC);
		add(rispostaD);
		domandaField.setBounds(52, 12, 335, 35);
		rispostaA.setBounds(55, 62, 146, 16);
		rispostaC.setBounds(55, 93, 146, 16);
		rispostaB.setBounds(260, 62, 146, 16);
		rispostaD.setBounds(260, 93, 146, 16);
		reset();
		registerListeners();
	}

	/**
	 *Accende la risposta selezionata.
	 * 
	 * @param risposta
	 *            numero da (1..4) della risposta.
	 */
	private void accendi(int risposta) {
		if ((!flag.isAnswerSelected()) && (rispostaSelezionata == 0)
				&& !(risposta == 0)) {
			switch (risposta) {
			case 1: {
				rispostaA.setBackground(Color.orange);
				rispostaA.setForeground(Color.black);
				rispostaSelezionata = 1;
				break;
			}
			case 2: {
				rispostaB.setBackground(Color.orange);
				rispostaB.setForeground(Color.black);
				rispostaSelezionata = 2;
				break;
			}
			case 3: {
				rispostaC.setBackground(Color.orange);
				rispostaC.setForeground(Color.black);
				rispostaSelezionata = 3;
				break;
			}
			case 4: {
				rispostaD.setBackground(Color.orange);
				rispostaD.setForeground(Color.black);
				rispostaSelezionata = 4;
			}
			}
			playSound("/data/accendiamo.wav");
			flag.setAnswerSelected(true);
		} else if (flag.isAnswerSelected() && (rispostaSelezionata != 0)
				&& (risposta == 0)) {
			rispostaA.setBackground(Color.black);
			rispostaA.setForeground(Color.white);
			rispostaB.setBackground(Color.black);
			rispostaB.setForeground(Color.white);
			rispostaC.setBackground(Color.black);
			rispostaC.setForeground(Color.white);
			rispostaD.setBackground(Color.black);
			rispostaD.setForeground(Color.white);
			rispostaSelezionata = 0;
			flag.setAnswerSelected(false);
		}
	}

	/**
	 *Controlla se la riposta accesa e' uguale a quella giusta.
	 * 
	 * @return true se la risposta selezionata e' giusta.
	 */
	public boolean check() {
		flag.setAnswerSelected(false);
		switch (rispostaGiusta) {
		case 1: {
			rispostaA.setBackground(Color.green);
			rispostaA.setForeground(Color.white);
			return (rispostaSelezionata == rispostaGiusta);
		}
		case 2: {
			rispostaB.setBackground(Color.green);
			rispostaB.setForeground(Color.white);
			return (rispostaSelezionata == rispostaGiusta);
		}
		case 3: {
			rispostaC.setBackground(Color.green);
			rispostaC.setForeground(Color.white);
			return (rispostaSelezionata == rispostaGiusta);
		}
		case 4: {
			rispostaD.setBackground(Color.green);
			rispostaD.setForeground(Color.white);
			return (rispostaSelezionata == rispostaGiusta);
		}
		}
		flag.setAnswerSelected(true);
		return false;
	}

	/**
	 * 
	 *
	 */
	protected void createTextAreas() {
		domandaField = new JTextArea(2, 40);
		domandaField.setEditable(false);
		domandaField.setFont(FONT);
		domandaField.setLineWrap(true);
		domandaField.setWrapStyleWord(true);
		domandaField.setToolTipText("Clicca per spegnere una risposta accesa");
		rispostaA = new JTextArea(1, 18);
		rispostaA.setEditable(false);
		rispostaA.setFont(FONT);
		rispostaA.setToolTipText("Clicca per accenderla");
		rispostaB = new JTextArea(1, 18);
		rispostaB.setEditable(false);
		rispostaB.setFont(FONT);
		rispostaB.setToolTipText("Clicca per accenderla");
		rispostaC = new JTextArea(1, 18);
		rispostaC.setEditable(false);
		rispostaC.setFont(FONT);
		rispostaC.setToolTipText("Clicca per accenderla");
		rispostaD = new JTextArea(1, 18);
		rispostaD.setEditable(false);
		rispostaD.setFont(FONT);
		rispostaD.setToolTipText("Clicca per accenderla");
	}

	/**
	 * Cancella due risposte
	 * 
	 * @param rispElim
	 */
	public void eliminaDue(int[] rispElim) {
		for (int i = 0; i < 2; i++) {
			int n = rispElim[i];
			switch (n) {
			case 1: {
				rispostaA.setText("");
				break;
			}
			case 2: {
				rispostaB.setText("");
				break;
			}
			case 3: {
				rispostaC.setText("");
				break;
			}
			case 4: {
				rispostaD.setText("");
				break;
			}
			}
		}
	}

	/**
	 *Il testo della domanda e le risposte nei riquadri.
	 * 
	 * @param dom
	 *            domanda da scrivere sul pannello.
	 */
	public void insertDomanda(Domanda dom) {
		if (!flag.isQuestionWrited()) {
			domandaField.setText(dom.getTesto());
			rispostaA.setText(dom.getRispostaA());
			rispostaB.setText(dom.getRispostaB());
			rispostaC.setText(dom.getRispostaC());
			rispostaD.setText(dom.getRispostaD());
			rispostaGiusta = dom.getRispostaGiusta();
			flag.setQuestionWrited(true);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent mevt) {
		Object obj = mevt.getSource();
		if (flag.isQuestionWrited()) {
			if (obj == rispostaA) {
				accendi(1);
			} else if (obj == rispostaB) {
				accendi(2);
			} else if (obj == rispostaC) {
				accendi(3);
			} else if (obj == rispostaD) {
				accendi(4);
			} else if (obj == domandaField) {
				accendi(0);
			} else {
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent mevt) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent mevt) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent mevt) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent mevt) {
	}

	/**
	 * Carica l'immagine di background
	 * 
	 */
	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawImage(img, 0, 0, this);
	}

	/**
	 * Riproduce gli effetti musicali.
	 * 
	 * @param musica
	 *            file .wav da suonare
	 */
	protected void playSound(String suono) {
		if (flag.isSoundEnabled()) {
			InputStream is = getClass().getResourceAsStream(suono);
			try {
				AudioFileFormat aff = AudioSystem.getAudioFileFormat(is);
				AudioInputStream ais = AudioSystem.getAudioInputStream(is);
				AudioFormat af = aff.getFormat();
				DataLine.Info info = new DataLine.Info(Clip.class, ais
						.getFormat(), ((int) ais.getFrameLength() * af
						.getFrameSize()));
				ol = (Clip) AudioSystem.getLine(info);
				ol.open(ais);
				ol.loop(0);
			} catch (Exception e) {
				e.printStackTrace();
				flag.setSoundEnabled(false);
			}

		} else {
			ol = null;
		}
	}

	/**
	 * 
	 *
	 */
	protected void registerListeners() {
		domandaField.addMouseListener(this);
		rispostaA.addMouseListener(this);
		rispostaB.addMouseListener(this);
		rispostaC.addMouseListener(this);
		rispostaD.addMouseListener(this);
	}

	/**
	 *Resetta il pannello. Cancella le domande scritte.
	 * 
	 */
	public void reset() {
		flag.setAnswerSelected(false);
		domandaField.setBackground(Color.black);
		domandaField.setForeground(Color.white);
		rispostaA.setBackground(Color.black);
		rispostaA.setForeground(Color.white);
		rispostaB.setBackground(Color.black);
		rispostaB.setForeground(Color.white);
		rispostaC.setBackground(Color.black);
		rispostaC.setForeground(Color.white);
		rispostaD.setBackground(Color.black);
		rispostaD.setForeground(Color.white);
		domandaField.setText("");
		rispostaA.setText("");
		rispostaB.setText("");
		rispostaC.setText("");
		rispostaD.setText("");
		rispostaSelezionata = 0;
	}

	/**
	 * 
	 *
	 */
	protected void setupComponents() {
		setBackground(Color.black);
		Toolkit tk = Toolkit.getDefaultToolkit();
		img = tk.getImage(getClass().getResource("/data/maschera.png"));
		createTextAreas();
	}
}
