package milionario;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Iterator;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import milionario.gui.MoneyPanel;
import milionario.gui.SplashWindow;

/**
 * Crea tutte le finestre ed ascolta gli eventi.
 * 
 * @author Loris Teodorani & Andrea Urbini
 * @version 4.8
 */

public class Milionario implements ActionListener {

	private Domanda domanda;

	private boolean aiuto5050Usato = false;
	private final Icon ERROR_ICON = new ImageIcon(getClass().getResource(
			"/data/error.png"));
	private final Icon INFO_ICON = new ImageIcon(getClass().getResource(
			"/data/info.png"));

	private boolean aiutoTelefonoUsato = false;
	private boolean aiutoPubblicoUsato = false;
	private boolean aiutoSwitchUsato = false;
	private int valore = 0;
	private int[] domandeFatte;
	private Clip ol;
	private PercorsoPanel perPanel;
	private DomandaPanel domPanel;
	private MoneyPanel monPanel;
	private PercentualiPanel pubPanel;
	private CountDownAgent telFrame;
	private JFrame domFrame, perFrame;
	private JDialog monFrame, pubFrame;
	private ArrayList<Domanda> vettoreDomande;
	private int[] domandePerLivello;
	private FlagRegister flag;
	private final String[] defaultJackpotList = new String[] { "0", "50",
			"100", "200", "300", "500", "1.000", "2.000", "4.000", "8.000",
			"16.000", "35.000", "70.000", "150.000", "300.000", "1 MILIONE" };
	private String[] jackpotList = new String[16];

	/**
	 * Costruttore
	 * 
	 * @param musicEnabled
	 *            true per abilitare la riproduzione della musica di sottofondo.
	 *@param soundEnabled
	 *            true per abilitare gli effetti sonori.
	 */

	public Milionario(boolean musicEnabled, boolean soundEnabled) {
		SplashWindow splash = new SplashWindow();
		splash.setVisible(true);
		flag = new FlagRegister();
		flag.setMusicEnabled(musicEnabled);
		flag.setSoundEnabled(soundEnabled);
		splash.setProgress(30);
		try {
			loadVettoreDomande();
			loadNumeroDomande();
		} catch (Exception e) {
			splash.setVisible(false);
			e.printStackTrace();
			JOptionPane
					.showMessageDialog(
							null,
							"File delle domande non trovato.\nIl programma verra' terminato",
							"ERRORE IRREVERSIBILE!", JOptionPane.ERROR_MESSAGE,
							ERROR_ICON);
			System.exit(-1);
		}
		splash.setProgress(40);
		loadJackpotList("data/jackpot.xml");
		saveJackpotList("data/jackpot.xml");
		splash.setProgress(60);
		setupComponents();
		splash.setProgress(80);
		createDialogs();
		splash.setProgress(100);
		createFrames();
		registerListener();
		reset();
		splash.setVisible(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent actionEvent) {
		String actionName = actionEvent.getActionCommand();
		if (actionName == "Domanda") {
			if (!flag.isQuestionWrited()) {
				if (valore < jackpotList.length - 1) {
					monFrame.setVisible(false);
					pubFrame.setVisible(false);
					domPanel.reset();
					playSound("/data/dumdum.wav");
					do {
					} while ((ol != null) && (ol.isRunning()));
					domanda = estraiDomanda();
					domPanel.insertDomanda(domanda);
					domFrame.requestFocus();
				}
			}
		} else if (actionName == "Resetta") {
			reset();
		} else if (actionName == "?") {
			JOptionPane.showMessageDialog(null, "Il Milionario 4.8" + "\n"
					+ "by Loris Teodorani e Andrea Urbini 2009" + "\n"
					+ "http://www.urbo.altervista.org", "About...",
					JOptionPane.INFORMATION_MESSAGE, INFO_ICON);
		} else if (actionName == "50:50" && !flag.isAnswerSelected()) {
			if (!aiuto5050Usato && flag.isQuestionWrited()) {
				aiuto5050Usato = true;
				perPanel.disable5050();
				domPanel.eliminaDue(domanda.getRispElimComp());
				domFrame.requestFocus();
			}
		} else if (actionName == " Tel ") {
			if (!aiutoTelefonoUsato && flag.isQuestionWrited()
					&& !flag.isAnswerSelected()) {
				perPanel.disableTel();
				aiutoTelefonoUsato = true;
				telFrame.start();
				domFrame.requestFocus();
			}
		} else if (actionName == "Pubbl") {
			if (!aiutoPubblicoUsato && flag.isQuestionWrited()
					&& !flag.isAnswerSelected()) {
				perPanel.disablePub();
				aiutoPubblicoUsato = true;
				playSound("/data/aiutoPub.wav");
				do {
				} while ((ol != null) && (ol.isRunning()));
				pubPanel.setPercentuali(domanda.getPercentuali());
				pubPanel.repaint();
				pubFrame.setVisible(true);
				domFrame.requestFocus();
			}
		} else if (actionName == "Switch") {
			if (!aiutoSwitchUsato && flag.isQuestionWrited()
					&& !flag.isAnswerSelected() && valore > 4) {
				perPanel.disableSwitch();
				aiutoSwitchUsato = true;
				flag.setQuestionWrited(false);
				domPanel.reset();
				domanda = estraiDomanda();
				domPanel.insertDomanda(domanda);
				domPanel.repaint();
				domFrame.requestFocus();
			}
		} else if (actionName == "Risposta definitiva") {
			if (flag.isQuestionWrited() && flag.isAnswerSelected()) {
				if (domPanel.check()) {
					// l'utente ha indovinato
					aumentaValore();
					playSound("/data/applauso.wav");
					flag.setQuestionWrited(false);
				} else {
					// l'utente ha sbagliato
					aiuto5050Usato = true;
					aiutoTelefonoUsato = true;
					aiutoPubblicoUsato = true;
					if (valore < 5) {
						valore = 0;
					} else if (valore < 10) {
						valore = 5;
					} else {
						valore = 10;
					}
				}
				mostraVincita();
			}
		}
	}

	/**
	 * 
	 *
	 */
	protected void aumentaValore() {
		perPanel.aumentaValore();
		valore++;
	}

	/**
	 * Crea le finestre secondarie di gioco
	 * 
	 */
	protected void createDialogs() {
		monFrame = new JDialog();
		monFrame.setTitle("Montepremi");
		monFrame.setBounds(0, 290, 445, 90);
		monFrame.setVisible(false);
		monFrame.getContentPane().add(monPanel);
		pubFrame = new JDialog();
		pubFrame.setTitle("Pubblico");
		pubFrame.setBounds(200, 0, 170, 250);
		pubFrame.setVisible(false);
		pubFrame.getContentPane().add(pubPanel);
	}

	/**
	 * Crea le finestre principali di gioco.
	 */
	protected void createFrames() {
		domFrame = new JFrame("Domanda");
		domFrame.setBounds(0, 380, 445, 160);
		domFrame.getContentPane().add(domPanel);
		domFrame.setResizable(false);
		domFrame.setVisible(true);
		perFrame = new JFrame("Percorso");
		perFrame.setBounds(480, 0, 310, 520);
		perFrame.getContentPane().add(perPanel);
		perFrame.setResizable(false);
		perFrame.setVisible(true);
	}

	/**
	 * Estra la domanda
	 * 
	 * @return domanda estratta.
	 */
	protected Domanda estraiDomanda() {
		int n = 0;
		do {
			if (valore <= 5) {
				n = 10001 + (int) (Math.random() * (domandePerLivello[0]));
			} else if (valore <= 10) {
				n = 20001 + (int) (Math.random() * (domandePerLivello[1]));
			} else {
				n = 30001 + (int) (Math.random() * (domandePerLivello[2]));
			}
		} while (searchNumber(n));
		domandeFatte[valore] = n;
		return searchDomanda(n);
	}

	private boolean loadJackpotList(String jackpotListFilename) {
		FileInputStream in = null;
		try {
			in = new FileInputStream(jackpotListFilename);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			jackpotList = defaultJackpotList;
			return false;
		}
		XMLDecoder dec = new XMLDecoder(in);
		try {
			jackpotList = (String[]) dec.readObject();
		} catch (Exception e) {
			e.printStackTrace();
			jackpotList = defaultJackpotList;
			return false;
		}
		return true;
	}

	/**
	 * Carica il file contenente il numero di domande per livello.
	 * 
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	protected void loadNumeroDomande() throws IOException,
			ClassNotFoundException {
		FileInputStream fis = new FileInputStream("data/domande.urb.nbr");
		ObjectInputStream ois = new ObjectInputStream(fis);
		domandePerLivello = (int[]) ois.readObject();
		ois.close();
	}

	/**
	 * Carica il file con le domande del gioco.
	 * 
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	protected void loadVettoreDomande() throws IOException,
			ClassNotFoundException {
		FileInputStream fis = new FileInputStream("data/domande.urb.cry");
		ObjectInputStream ois = new ObjectInputStream(fis);
		vettoreDomande = (ArrayList<Domanda>) ois.readObject();
		ois.close();
		fis.close();
	}

	/**
	 *Visualizza il montepremi conquistato
	 * 
	 */
	private void mostraVincita() {
		monPanel.repaint(valore);
		monFrame.setVisible(true);
	}

	/**
	 *Riproduce un suono memorizzato in un file .wav
	 * 
	 * @param suono
	 *            suono da riprodurre
	 */
	protected void playSound(String suono) {
		if (flag.isSoundEnabled()) {
			InputStream is = getClass().getResourceAsStream(suono);
			try {
				AudioFileFormat aff = AudioSystem.getAudioFileFormat(is);
				AudioInputStream ais = AudioSystem.getAudioInputStream(is);
				AudioFormat af = aff.getFormat();
				DataLine.Info info = new DataLine.Info(Clip.class, ais
						.getFormat(), ((int) ais.getFrameLength() * af
						.getFrameSize()));
				ol = (Clip) AudioSystem.getLine(info);
				ol.open(ais);
				ol.loop(0);
			} catch (Exception e) {
				e.printStackTrace();
				flag.setSoundEnabled(false);
				perPanel.disableSoundEffects();
			}

		} else {
			ol = null;
		}
	}

	/**
	 * 
	 *
	 */
	protected void registerListener() {
		perFrame.addWindowListener(new Terminator(this));
		domFrame.addWindowListener(new Terminator(this));
	}

	/**
	 * 
	 *
	 */
	protected void reset() {
		monFrame.setVisible(false);
		pubFrame.setVisible(false);
		domPanel.reset();
		perPanel.reset();
		aiuto5050Usato = false;
		aiutoTelefonoUsato = false;
		aiutoPubblicoUsato = false;
		aiutoSwitchUsato = false;
		flag.setQuestionWrited(false);
		valore = 0;
		domandeFatte = new int[15];
	}

	private boolean saveJackpotList(String jackpotListFilename) {
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(jackpotListFilename);
		} catch (FileNotFoundException fnfe) {
			fnfe.printStackTrace();
			return false;
		}
		XMLEncoder en = new XMLEncoder(out);
		en.writeObject(jackpotList);
		en.flush();
		en.close();
		return true;
	}

	/**
	 * Cerca la domanda nel vettore delle domande
	 * 
	 * @param n
	 *            numero identificativo della domanda
	 * @return la domanda, null se la domanda non e' stata trovata.
	 */
	protected Domanda searchDomanda(int n) {
		Iterator<Domanda> i = vettoreDomande.iterator();
		while (i.hasNext()) {
			Domanda domanda = i.next();
			if (domanda.getIdNumber() == n) {
				return domanda;
			}
		}
		return null;
	}

	/**
	 * Controlla che il numero identificativo della domanda estratta non sia
	 * gia' stato estratto
	 * 
	 * @param n
	 *            numero della domanda estratta
	 * @return true se la domanda era gia' stata estratta
	 */
	private boolean searchNumber(int n) {
		for (int i = 0; i < domandeFatte.length; i++) {
			if (n == domandeFatte[i]) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Crea i pannelli di gioco.
	 */
	protected void setupComponents() {
		perPanel = new PercorsoPanel(flag, jackpotList, this);
		domPanel = new DomandaPanel(flag);
		monPanel = new MoneyPanel(jackpotList);
		telFrame = new CountDownAgent();
		pubPanel = new PercentualiPanel();
	}
}
