package milionario;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.URL;

import javax.sound.sampled.LineUnavailableException;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;

/**
 * Visualizza la scalata al milione e contiene il pulsanti di gioco
 * 
 * @author Andrea Urbini
 * @version 5.0
 */

public class PercorsoPanel extends JPanel implements ItemListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6651847482308275499L;
	private URL adrComputer = getClass().getResource("/data/computer.png");
	private URL adrPubblico = getClass().getResource("/data/pubblico.png");
	private URL adrSwitch = getClass().getResource("/data/switch.png");
	private URL adrTelefono = getClass().getResource("/data/telefono.png");
	public JButton dom, aiuto5050, aiutoTelefono, aiutoPubblico, aiutoSwitch,
			risposta, resetta, about;
	private JCheckBox effetti;
	private FlagRegister flag = null;
	private JLabel imgComputer, imgTelefono, imgPubblico, imgSwitch;
	private String[] jackpotList = null;
	private MusicPlayer mp = null;
	private JCheckBox musica;
	private ActionListener owner = null;
	private JTable pathTable = null;
	private int valore = 15;

	/**
	 * Costruttore
	 * 
	 * @param owner
	 */
	public PercorsoPanel(FlagRegister flag, String[] jackpotList,
			ActionListener owner) {
		super();
		this.flag = flag;
		this.jackpotList = jackpotList;
		this.owner = owner;
		setupComponents();
		setBackground(Color.black);
		Box icone = new Box(BoxLayout.X_AXIS);
		icone.add(imgComputer);
		icone.add(imgTelefono);
		icone.add(imgPubblico);
		icone.add(imgSwitch);
		Box tabella = new Box(BoxLayout.X_AXIS);
		tabella.add(Box.createHorizontalStrut(40));
		tabella.add(getPathTable());
		Box bottoni1 = new Box(BoxLayout.X_AXIS);
		bottoni1.add(dom);
		bottoni1.add(Box.createHorizontalStrut(195));
		Box bottoni2 = new Box(BoxLayout.X_AXIS);
		bottoni2.add(aiuto5050);
		bottoni2.add(Box.createHorizontalStrut(5));
		bottoni2.add(aiutoTelefono);
		bottoni2.add(Box.createHorizontalStrut(5));
		bottoni2.add(aiutoPubblico);
		bottoni2.add(Box.createHorizontalStrut(5));
		bottoni2.add(aiutoSwitch);
		Box bottoni3 = new Box(BoxLayout.X_AXIS);
		bottoni3.add(risposta);
		bottoni3.add(Box.createHorizontalStrut(18));
		bottoni3.add(resetta);
		bottoni3.add(about);
		Box sound = new Box(BoxLayout.X_AXIS);
		sound.add(effetti);
		sound.add(Box.createHorizontalStrut(70));
		sound.add(musica);
		Box colonna1 = new Box(BoxLayout.Y_AXIS);
		colonna1.add(Box.createVerticalStrut(30));
		colonna1.add(icone);
		colonna1.add(Box.createVerticalStrut(30));
		colonna1.add(tabella);
		colonna1.add(Box.createVerticalStrut(30));
		colonna1.add(bottoni1);
		colonna1.add(Box.createVerticalStrut(2));
		colonna1.add(bottoni2);
		colonna1.add(Box.createVerticalStrut(2));
		colonna1.add(bottoni3);
		colonna1.add(Box.createVerticalStrut(6));
		colonna1.add(sound);
		add(colonna1);
		registerListeners();
		if (flag.isMusicEnabled()) {
			try {
				mp.play();
			} catch (Exception e) {
				e.printStackTrace();
				flag.setMusicEnabled(false);
				musica.setSelected(false);
			}
		}
	}

	/**
	 * Alza il selettore del livello di gioco. La selezione sulla tabella
	 */

	public void aumentaValore() {
		valore--;
		getPathTable().setRowSelectionInterval(valore, valore);
	}

	protected void createButtons() {
		dom = createMilButton("Domanda");
		dom.setToolTipText("Passa alla prossima domanda");
		aiuto5050 = createMilButton("50:50");
		aiuto5050.setToolTipText("Chiedi l'aiuto del computer");
		aiutoTelefono = createMilButton(" Tel ");
		aiutoTelefono.setToolTipText("Chiedi l'aiuto da casa");
		aiutoPubblico = createMilButton("Pubbl");
		aiutoPubblico.setToolTipText("Chiedi l'aiuto del pubblico");
		aiutoSwitch = createMilButton("Switch");
		aiutoSwitch.setToolTipText("Cambia la domanda");
		risposta = createMilButton("Risposta definitiva");
		risposta.setToolTipText("La accendiamo?");
		resetta = createMilButton("Resetta");
		resetta.setToolTipText("Lasci il gioco");
		about = createMilButton("?");
		about.setToolTipText("About");
	}

	/**
	 * Crea e configura i due CheckBox presenti nel pannello. Servono per
	 * attivare e disattivare la musica di sottofondo e gli effetti sonori
	 */
	protected void createCheckBoxes() {
		effetti = new JCheckBox("Effetti sonori", flag.isSoundEnabled());
		effetti.setFont(new Font("SansSerif", Font.BOLD, 12));
		effetti.setBackground(Color.black);
		effetti.setForeground(new Color(35, 132, 169));
		effetti.setSelectedIcon(new ImageIcon(getClass().getResource(
				"/data/volume-high.png")));
		effetti
				.setIcon(new ImageIcon(getClass().getResource("/data/mute.png")));
		musica = new JCheckBox("Musica", flag.isMusicEnabled());
		musica.setFont(new Font("SansSerif", Font.BOLD, 12));
		musica.setBackground(Color.black);
		musica.setForeground(new Color(35, 132, 169));
		musica.setSelectedIcon(new ImageIcon(getClass().getResource(
				"/data/volume-high.png")));
		musica.setIcon(new ImageIcon(getClass().getResource("/data/mute.png")));
	}

	protected void createIcons() {
		imgComputer = new JLabel(new ImageIcon(adrComputer));
		imgTelefono = new JLabel(new ImageIcon(adrTelefono));
		imgPubblico = new JLabel(new ImageIcon(adrPubblico));
		imgSwitch = new JLabel(new ImageIcon(adrSwitch));
	}

	/**
	 * Crea e configura un oggetto JButton
	 * 
	 * @param nome
	 *            Etichetta sul bottone
	 * @return un oggetto di tipo JButton
	 */
	protected JButton createMilButton(String nome) {
		JButton b = new JButton(nome);
		b.setForeground(new Color(35, 132, 169));
		b.setBackground(Color.BLACK);
		b.setFont(new Font("SansSerif", Font.BOLD, 12));
		return b;
	}

	public void disable5050() {
		adrComputer = getClass().getResource("/data/computerUsato.png");
		imgComputer.setIcon(new ImageIcon(adrComputer));
		repaint();
	}

	public void disablePub() {
		adrPubblico = getClass().getResource("/data/pubblicoUsato.png");
		imgPubblico.setIcon(new ImageIcon(adrPubblico));
		repaint();
	}

	public void disableSoundEffects() {
		flag.setSoundEnabled(false);
		effetti.setEnabled(false);
	}

	public void disableSwitch() {
		adrSwitch = getClass().getResource("/data/switchUsato.png");
		imgSwitch.setIcon(new ImageIcon(adrSwitch));
		repaint();
	}

	public void disableTel() {
		adrTelefono = getClass().getResource("/data/telefonoUsato.png");
		imgTelefono.setIcon(new ImageIcon(adrTelefono));
		repaint();
	}

	protected JTable getPathTable() {
		if (pathTable == null) {
			String[][] tappe = new String[jackpotList.length - 1][2];
			for (int i = jackpotList.length - 1; i > 0; i--) {
				tappe[jackpotList.length - 1 - i] = new String[] { "" + i,
						jackpotList[i] };
			}
			pathTable = new JTable(tappe, new String[] { "", "" });
			pathTable.setFont(new Font("SansSerif", Font.BOLD, 16));
			pathTable.setEnabled(false);
			pathTable.setShowGrid(false);
			pathTable.setGridColor(Color.orange);
			pathTable.setBackground(Color.black);
			pathTable.setForeground(Color.white);
			pathTable.setSelectionBackground(Color.orange);
			pathTable.setSelectionForeground(Color.white);
		}
		return pathTable;
	}

	/**
	 * Ascolta gli eventi dei CheckBox che abilitano/disabilitano la musica e
	 * gli effetti sonori.
	 */

	public void itemStateChanged(ItemEvent ie) {
		if (ie.getSource().equals(musica)) {
			flag.setMusicEnabled((musica.isSelected()));
			if (musica.isSelected()) {
				try {
					mp.play();
				} catch (Exception e) {
					e.printStackTrace();
					musica.setSelected(false);
					flag.setMusicEnabled(false);
				}
			} else {
				mp.pause();
			}
		} else {
			flag.setSoundEnabled((effetti.isSelected()));
		}
	}

	protected void registerListeners() {
		dom.addActionListener(owner);
		aiuto5050.addActionListener(owner);
		aiutoTelefono.addActionListener(owner);
		aiutoPubblico.addActionListener(owner);
		aiutoSwitch.addActionListener(owner);
		risposta.addActionListener(owner);
		resetta.addActionListener(owner);
		about.addActionListener(owner);
		effetti.addItemListener(this);
		musica.addItemListener(this);
	}

	/**
	 * Resetta la grafica e azzera il percorso fatto
	 */

	public void reset() {
		adrComputer = getClass().getResource("/data/computer.png");
		adrTelefono = getClass().getResource("/data/telefono.png");
		adrPubblico = getClass().getResource("/data/pubblico.png");
		adrSwitch = getClass().getResource("/data/switch.png");
		imgComputer.setIcon(new ImageIcon(adrComputer));
		imgTelefono.setIcon(new ImageIcon(adrTelefono));
		imgPubblico.setIcon(new ImageIcon(adrPubblico));
		imgSwitch.setIcon(new ImageIcon(adrSwitch));
		getPathTable().removeRowSelectionInterval(0, 14);
		valore = 15;
	}

	protected void setupComponents() {
		createIcons();
		createButtons();
		createCheckBoxes();
		try {
			mp = new MusicPlayer();
		} catch (LineUnavailableException lue) {
			lue.printStackTrace();
			musica.setEnabled(false);
			musica.setSelected(false);
			flag.setMusicEnabled(false);
		}
	}

}
